import os
from socket import gethostbyname
from subprocess import check_output

from charmhelpers.core.hookenv import (
    related_units,
    relation_get,
    relation_ids,
    unit_get
)

from charmhelpers.core.host import service_restart

from charmhelpers.core.templating import render

def analytics_api_ctx():
    return { "host_ip": gethostbyname(unit_get("private-address")) }

def cassandra_ctx():
    return { "cassandra_servers": [ gethostbyname(relation_get("private-address", unit, rid))
                                    + ":" + port
                                    for rid in relation_ids("cassandra")
                                    for unit, port in
                                    ((unit, relation_get("port", unit, rid)) for unit in related_units(rid))
                                    if port ] }

def discovery_ctx():
    ctxs = [ { "disc_server_ip": gethostbyname(relation_get("private-address", unit, rid)),
               "disc_server_port": port }
             for rid in relation_ids("contrail-discovery")
             for unit, port in
             ((unit, relation_get("port", unit, rid)) for unit in related_units(rid))
             if port ]
    return ctxs[0] if ctxs else {}

def fix_permissions():
    os.chmod("/etc/contrail", 0755)
    os.chown("/etc/contrail", 0, 0)

def fix_services():
    # redis listens on localhost by default
    check_output(["sed", "-i", "-e",
                  "s/^bind /# bind /",
                  "/etc/redis/redis.conf"])
    service_restart("redis-server")

def write_analytics_api_config():
    ctx = {}
    ctx.update(analytics_api_ctx())
    ctx.update(cassandra_ctx())
    ctx.update(discovery_ctx())
    render("contrail-analytics-api.conf",
           "/etc/contrail/contrail-analytics-api.conf", ctx)

def write_collector_config():
    ctx = {}
    ctx.update(cassandra_ctx())
    ctx.update(discovery_ctx())
    render("contrail-collector.conf",
           "/etc/contrail/contrail-collector.conf", ctx)

def write_query_engine_config():
    ctx = {}
    ctx.update(cassandra_ctx())
    ctx.update(discovery_ctx())
    render("contrail-query-engine.conf",
           "/etc/contrail/contrail-query-engine.conf", ctx)
